import os
import sys

from .arg_parse import parse
from .localisation import ServiceVocabulary
from .session_cache import OpenSessions
from .telegram import Service, handlers_register
from .session_cache import background_expired_sessions_handler
from .google_sheets import DB_AS_GoogleSheet
from .utils import create_logger
from . import PACKAGE_DIR

logger = None


def main(*callback_list):
    try:
        OpenSessions.start()
        DB_AS_GoogleSheet.init(table_url=os.getenv('TABLE_URL'),
                               table_sheet=os.getenv('TABLE_SHEET'),
                               drive_folder_id=os.getenv('TABLE_FOLDER', None),
                               service_account_file=os.path.join(os.getenv('CREDENTIALS'), os.getenv('SECRET_FILE')))
        ServiceVocabulary.load(os.path.join(PACKAGE_DIR, 'configuration', 'localisation.yaml'))
        handlers_register(Service.dp)
        Service.start(start_up=callback_list)
    except KeyboardInterrupt:
        logger.info(f"User keyword interrupt")
    except Exception as e:
        logger.error(f"Unexpected: {e}")
    finally:
        Service.stop()


if __name__ == '__main__':
    arguments = parse(sys.argv[1:])
    logger = create_logger('Bot', arguments.debug)
    main(background_expired_sessions_handler(Service.bot))
